--
-- Rubbish by Ludovicus (LuisOrtiz@Verizon.NET)
--
-- Field Bag Management Functions
--
-- This addon is dedidicated to Harry Wasylyk & Larry Hansen, the Canadian inventors of the disposable green polyethylene garbage bag.l

local eventFrame = CreateFrame("Frame")

function Rubbish:Field_Init()
    Rubbish.TodaysTrash = {}
	Rubbish:dprint("Rubbish:Field_Init()")
	Rubbish:ScanAllBags()
	Rubbish:RegisterEvent("BAG_UPDATE")
	eventFrame:SetScript("OnUpdate",Rubbish_OnUpdate)
	StaticPopupDialogs["DELETE_ITEM"].OnAccept= function () local type, data, subType = GetCursorInfo();  if type == "item" then Rubbish.TodaysTrash[data] = true; end ; DeleteCursorItem(); end
end

function Rubbish:Field_Disable()
    Rubbish.TodaysTrash = {}
	Rubbish:dprint("Rubbish:Field_Disable()")
	Rubbish:UnregisterEvent("BAG_UPDATE")
	eventFrame:SetScript("OnUpdate",nil)
end


function Rubbish:ScanAllBags()
--	Rubbish:dprint("Rubbish:ScanAllBags()")
	Rubbish.Bag = {}
	Rubbish.Junk = {}
	Rubbish.FreeSlots = 0
	for bag=BACKPACK_CONTAINER, NUM_BAG_SLOTS do
		Rubbish.Bag[bag] = {}
		Rubbish:ScanBag(bag)
	end
end

function Rubbish:ScanBag(bag)
--	Rubbish:dprint("Rubbish:ScanBag(",bag,")")
	Rubbish.Bag[bag].FreeSlots = GetContainerNumFreeSlots(bag)
	Rubbish.Bag[bag].Slots = GetContainerNumSlots(bag)
	Rubbish.Bag[bag].Dirty = false
	
	local trimmedJunk = {}
	for offset,data in ipairs(Rubbish.Junk) do
		if data[1] ~= bag then
			table.insert(trimmedJunk,data)
		end
	end
	Rubbish.Junk = trimmedJunk
--	Rubbish:dprint("Rubbish:ScanBag",bag,"has",Rubbish.Bag[bag].Slots,"slots");
	if Rubbish.Bag[bag].Slots then
		for slot=1,Rubbish.Bag[bag].Slots do
			local link=GetContainerItemLink(bag,slot)
			if link then
				local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount,itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(link)
				local texture, itemCount, locked, quality, readable = GetContainerItemInfo(bag, slot);
--				Rubbish:dprint("Rubbish:ScanBag","slot=",slot,"itemName=",itemName,"itemCount=", itemCount, "itemType=",itemType, "itemRarity=",itemRarity, "itemSellPrice=", itemSellPrice)
				if itemRarity == 0 then
					table.insert(Rubbish.Junk,{bag,slot, itemCount * itemSellPrice, itemName})
--					Rubbish:dwarn("ScanBag: Added",itemName,"to list of junk")
				end
			end
		end
	end
end

function Rubbish:FreeSlotsCount()
	Rubbish.FreeSlots = 0
	for bag=BACKPACK_CONTAINER, NUM_BAG_SLOTS do
		if Rubbish.Bag[bag].FreeSlots then
			Rubbish.FreeSlots = Rubbish.FreeSlots + Rubbish.Bag[bag].FreeSlots
		end
	end
	return Rubbish.FreeSlots
end
	
function Rubbish:Dispose(bag,slot)
--	Rubbish:dwarn("Rubbish:Dispose",bag,slot)
	local link=GetContainerItemLink(bag,slot)
	local itemStart, itemEnd = string.find(link,"item:(%d+):")
	local itemID = tonumber(string.sub(link,itemStart+5,itemEnd-1))
	local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount,itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(link)
	Rubbish:dprint("Rubbish:Dispose of", itemLink,itemID )
	ClearCursor()
	PickupContainerItem(bag,slot)
	if Rubbish.TodaysTrash[itemID] then
	    Rubbish:say("Rubbish: Silent Trash of",itemID)
	    DeleteCursorItem()
	    return
	end
	StaticPopup_Show("DELETE_ITEM",itemName)
end

function Rubbish:FreeSlotCheck(slots_free)
	for bag=BACKPACK_CONTAINER, NUM_BAG_SLOTS do
		if Rubbish.Bag[bag].Dirty then
			Rubbish:ScanBag(bag)
		end
	end
	if Rubbish:FreeSlotsCount() < slots_free then
		local trash = nil
--		Rubbish:dwarn("Rubbish: Trash list of len",#Rubbish.Junk)
		for offset,data in ipairs(Rubbish.Junk) do
--			Rubbish:dwarn("Rubbish: Maybe Trash", data[4], data[3])
			if trash ~= nil and trash[3] > data[3] then
				trash = data
--				Rubbish:dwarn("Rubbish: Yes");
			elseif trash == nil then
--				Rubbish:dwarn("Rubbish: First");
				trash = data
			end
		end
		if trash then
			Rubbish:Dispose(trash[1],trash[2])
		else
--			Rubbish:dwarn("Rubbish:BAG_UPDATE no trash found!")
		end
	else
--		Rubbish:dprint("Rubbish:BAG_UPDATE",Rubbish.FreeSlots,"Free")
	end
end

function Rubbish_OnUpdate(self,elapsed)
	if Rubbish.TimeSinceLastBagUpdate ~= nil then
		Rubbish.TimeSinceLastBagUpdate = Rubbish.TimeSinceLastBagUpdate + elapsed
  		if Rubbish.TimeSinceLastBagUpdate and Rubbish.TimeSinceLastBagUpdate > 0.125 then
  			Rubbish.TimeSinceLastBagUpdate = nil
  			Rubbish:FreeSlotCheck(2)
  		end
  	end
end

function Rubbish:BAG_UPDATE(event,bag)
--	Rubbish:dwarn("BAG_UPDATE",type(bag),bag)
	if bag < 0 then
		return
	end
	if Rubbish.Bag == nil then
		Rubbish.Bag = {}
	end
	if Rubbish.Bag[bag] == nil then
		Rubbish.Bag[bag] = {}
	end
	Rubbish.Bag[bag].Dirty = true
	Rubbish.Bag[bag].FreeSlots = GetContainerNumFreeSlots(bag)
	Rubbish.TimeSinceLastBagUpdate = 0;
end














